/*============================================================================
Filename : touch.c
Project : QTouch Modular Library
Purpose : Provides Initialization, Processing and ISR handler of touch library,
          Simple API functions to get/set the key touch parameters from/to the
          touch library data structures

This file is part of QTouch Modular Library Release 7.2 application.

Important Note: Do not edit this file manually.
                Use QTouch Configurator within Atmel Start to apply any
                modifications to this file.

Usage License: Refer license.h file for license information
Support: Visit http://www.microchip.com/support/hottopics.aspx
               to create MySupport case.

------------------------------------------------------------------------------
Copyright (c) 2020 Microchip. All rights reserved.
------------------------------------------------------------------------------
============================================================================*/
#ifndef TOUCH_C
#define TOUCH_C

#ifdef __XC8__
#warning "The library is not tested for XC8 compiler, use AVR-GCC compiler instead"
#endif
#ifdef __XC32__
#warning "The library is not tested for XC32 compiler, use ARM-GCC compiler instead"
#endif

/*----------------------------------------------------------------------------
 *     include files
 *----------------------------------------------------------------------------*/

#include <atmel_start.h>

#include "touch.h"
#include "license.h"

#include "datastreamer.h"

/*----------------------------------------------------------------------------
 *   prototypes
 *----------------------------------------------------------------------------*/

/*! \brief configure keys, wheels and sliders.
 */
static touch_ret_t touch_sensors_config(void);

/*! \brief Touch measure complete callback function example prototype.
 */
static void qtm_measure_complete_callback(void);

/*! \brief Touch Error callback function prototype.
 */
static void qtm_error_callback(uint8_t error);

/*----------------------------------------------------------------------------
 *     Global Variables
 *----------------------------------------------------------------------------*/
/* Acquisition Index */
volatile uint8_t acq_set = 1;

/* Flag to indicate time for touch measurement */
volatile uint8_t time_to_measure_touch_flag = 0;

/* post process request flag */
volatile uint8_t touch_postprocess_request = 0;

/* Measurement Done Touch Flag  */
volatile uint8_t measurement_done_touch = 0;

/* Error Handling */
uint8_t module_error_code = 0;

/*################################################################# */
/*######################## Configuration 1 ######################## */
/*################################################################# */

/* Acquisition module internal data - Size to largest acquisition set */
uint16_t touch_acq_signals_raw1[DEF_NUM_CHANNELS_1];

/* Acquisition set 1 - General settings */
qtm_acq_node_group_config_t ptc_qtlib_acq_gen1
    = {DEF_NUM_CHANNELS_1, DEF_SENSOR_TYPE_1, DEF_PTC_CAL_AUTO_TUNE_1, DEF_SEL_FREQ_INIT_1, DEF_PTC_INTERRUPT_PRIORITY_1};

/* Node status, signal, calibration values */
qtm_acq_node_data_t ptc_qtlib_node_stat1[DEF_NUM_CHANNELS_1];

/* Node configurations */
qtm_acq_samd20_node_config_t ptc_seq_node_cfg1[DEF_NUM_CHANNELS_1] = {NODE_0_PARAMS_1, NODE_1_PARAMS_1};

/* Container */
qtm_acquisition_control_t qtlib_acq_set1 = {&ptc_qtlib_acq_gen1, &ptc_seq_node_cfg1[0], &ptc_qtlib_node_stat1[0]};

/**********************************************************/
/*********** Frequency Hop Auto tune Module **********************/
/**********************************************************/

/* Buffer used with various noise filtering functions */
uint16_t noise_filter_buffer1[DEF_NUM_SENSORS_1 * NUM_FREQ_STEPS_1];
uint8_t  freq_hop_delay_selection1[NUM_FREQ_STEPS_1] = {DEF_MEDIAN_FILTER_FREQUENCIES_1};
uint8_t  freq_hop_autotune_counters1[NUM_FREQ_STEPS_1];

/* Configuration */
qtm_freq_hop_autotune_config_t qtm_freq_hop_autotune_config1 = {DEF_NUM_CHANNELS_1,
                                                                NUM_FREQ_STEPS_1,
                                                                &ptc_qtlib_acq_gen1.freq_option_select,
                                                                &freq_hop_delay_selection1[0],
                                                                DEF_FREQ_AUTOTUNE_ENABLE_1,
                                                                FREQ_AUTOTUNE_MAX_VARIANCE_1,
                                                                FREQ_AUTOTUNE_COUNT_IN_1};

/* Data */
qtm_freq_hop_autotune_data_t qtm_freq_hop_autotune_data1
    = {0, 0, &noise_filter_buffer1[0], &ptc_qtlib_node_stat1[0], &freq_hop_autotune_counters1[0]};

/* Container */
qtm_freq_hop_autotune_control_t qtm_freq_hop_autotune_control1
    = {&qtm_freq_hop_autotune_data1, &qtm_freq_hop_autotune_config1};

/**********************************************************/
/*********************** Keys Module **********************/
/**********************************************************/

/* Keys set 1 - General settings */
qtm_touch_key_group_config_t qtlib_key_grp_config_set1 = {DEF_NUM_SENSORS_1,
                                                          DEF_TOUCH_DET_INT_1,
                                                          DEF_MAX_ON_DURATION_1,
                                                          DEF_ANTI_TCH_DET_INT_1,
                                                          DEF_ANTI_TCH_RECAL_THRSHLD_1,
                                                          DEF_TCH_DRIFT_RATE_1,
                                                          DEF_ANTI_TCH_DRIFT_RATE_1,
                                                          DEF_DRIFT_HOLD_TIME_1,
                                                          DEF_REBURST_MODE_1};

qtm_touch_key_group_data_t qtlib_key_grp_data_set1;

/* Key data */
qtm_touch_key_data_t qtlib_key_data_set1[DEF_NUM_SENSORS_1];

/* Key Configurations */
qtm_touch_key_config_t qtlib_key_configs_set1[DEF_NUM_SENSORS_1] = {KEY_0_PARAMS_1, KEY_1_PARAMS_1};

/* Container */
qtm_touch_key_control_t qtlib_key_set1
    = {&qtlib_key_grp_data_set1, &qtlib_key_grp_config_set1, &qtlib_key_data_set1[0], &qtlib_key_configs_set1[0]};

/*################################################################# */
/*######################## Configuration 2 ######################## */
/*################################################################# */

/* Acquisition module internal data - Size to largest acquisition set */
uint16_t touch_acq_signals_raw2[DEF_NUM_CHANNELS_2];

/* Acquisition set 2 - General settings */
qtm_acq_node_group_config_t ptc_qtlib_acq_gen2
= {DEF_NUM_CHANNELS_2, DEF_SENSOR_TYPE_2, DEF_PTC_CAL_AUTO_TUNE_2, DEF_SEL_FREQ_INIT_2, DEF_PTC_INTERRUPT_PRIORITY_2};

/* Node status, signal, calibration values */
qtm_acq_node_data_t ptc_qtlib_node_stat2[DEF_NUM_CHANNELS_2];

/* Node configurations */
qtm_acq_samd20_node_config_t ptc_seq_node_cfg2[DEF_NUM_CHANNELS_2] = {NODE_0_PARAMS_2, NODE_1_PARAMS_2};

/* Container */
qtm_acquisition_control_t qtlib_acq_set2 = {&ptc_qtlib_acq_gen2, &ptc_seq_node_cfg2[0], &ptc_qtlib_node_stat2[0]};

/**********************************************************/
/*********** Frequency Hop Auto tune Module **********************/
/**********************************************************/

/* Buffer used with various noise filtering functions */
uint16_t noise_filter_buffer2[DEF_NUM_SENSORS_2 * NUM_FREQ_STEPS_2];
uint8_t  freq_hop_delay_selection2[NUM_FREQ_STEPS_2] = {DEF_MEDIAN_FILTER_FREQUENCIES_2};
uint8_t  freq_hop_autotune_counters2[NUM_FREQ_STEPS_2];

/* Configuration */
qtm_freq_hop_autotune_config_t qtm_freq_hop_autotune_config2 = {DEF_NUM_CHANNELS_2,
	NUM_FREQ_STEPS_2,
	&ptc_qtlib_acq_gen2.freq_option_select,
	&freq_hop_delay_selection2[0],
	DEF_FREQ_AUTOTUNE_ENABLE_2,
	FREQ_AUTOTUNE_MAX_VARIANCE_2,
FREQ_AUTOTUNE_COUNT_IN_2};

/* Data */
qtm_freq_hop_autotune_data_t qtm_freq_hop_autotune_data2
= {0, 0, &noise_filter_buffer2[0], &ptc_qtlib_node_stat2[0], &freq_hop_autotune_counters2[0]};

/* Container */
qtm_freq_hop_autotune_control_t qtm_freq_hop_autotune_control2
= {&qtm_freq_hop_autotune_data2, &qtm_freq_hop_autotune_config2};

/**********************************************************/
/*********************** Keys Module **********************/
/**********************************************************/

/* Keys set 2 - General settings */
qtm_touch_key_group_config_t qtlib_key_grp_config_set2 = {DEF_NUM_SENSORS_2,
	DEF_TOUCH_DET_INT_2,
	DEF_MAX_ON_DURATION_2,
	DEF_ANTI_TCH_DET_INT_2,
	DEF_ANTI_TCH_RECAL_THRSHLD_2,
	DEF_TCH_DRIFT_RATE_2,
	DEF_ANTI_TCH_DRIFT_RATE_2,
	DEF_DRIFT_HOLD_TIME_2,
DEF_REBURST_MODE_2};

qtm_touch_key_group_data_t qtlib_key_grp_data_set2;

/* Key data */
qtm_touch_key_data_t qtlib_key_data_set2[DEF_NUM_SENSORS_2];

/* Key Configurations */
qtm_touch_key_config_t qtlib_key_configs_set2[DEF_NUM_SENSORS_2] = {KEY_0_PARAMS_2, KEY_1_PARAMS_2};

/* Container */
qtm_touch_key_control_t qtlib_key_set2
= {&qtlib_key_grp_data_set2, &qtlib_key_grp_config_set2, &qtlib_key_data_set2[0], &qtlib_key_configs_set2[0]};

/*################################################################# */
/*######################## Configuration 3 ######################## */
/*################################################################# */

/* Acquisition module internal data - Size to largest acquisition set */
uint16_t touch_acq_signals_raw3[DEF_NUM_CHANNELS_3];

/* Acquisition set 3 - General settings */
qtm_acq_node_group_config_t ptc_qtlib_acq_gen3
= {DEF_NUM_CHANNELS_3, DEF_SENSOR_TYPE_3, DEF_PTC_CAL_AUTO_TUNE_3, DEF_SEL_FREQ_INIT_3, DEF_PTC_INTERRUPT_PRIORITY_3};

/* Node status, signal, calibration values */
qtm_acq_node_data_t ptc_qtlib_node_stat3[DEF_NUM_CHANNELS_3];

/* Node configurations */
qtm_acq_samd20_node_config_t ptc_seq_node_cfg3[DEF_NUM_CHANNELS_3] = {NODE_0_PARAMS_3, NODE_1_PARAMS_3};

/* Container */
qtm_acquisition_control_t qtlib_acq_set3 = {&ptc_qtlib_acq_gen3, &ptc_seq_node_cfg3[0], &ptc_qtlib_node_stat3[0]};

/**********************************************************/
/*********** Frequency Hop Auto tune Module **********************/
/**********************************************************/

/* Buffer used with various noise filtering functions */
uint16_t noise_filter_buffer3[DEF_NUM_SENSORS_3 * NUM_FREQ_STEPS_3];
uint8_t  freq_hop_delay_selection3[NUM_FREQ_STEPS_3] = {DEF_MEDIAN_FILTER_FREQUENCIES_3};
uint8_t  freq_hop_autotune_counters3[NUM_FREQ_STEPS_3];

/* Configuration */
qtm_freq_hop_autotune_config_t qtm_freq_hop_autotune_config3 = {DEF_NUM_CHANNELS_3,
	NUM_FREQ_STEPS_3,
	&ptc_qtlib_acq_gen3.freq_option_select,
	&freq_hop_delay_selection3[0],
	DEF_FREQ_AUTOTUNE_ENABLE_3,
	FREQ_AUTOTUNE_MAX_VARIANCE_3,
FREQ_AUTOTUNE_COUNT_IN_3};

/* Data */
qtm_freq_hop_autotune_data_t qtm_freq_hop_autotune_data3
= {0, 0, &noise_filter_buffer3[0], &ptc_qtlib_node_stat3[0], &freq_hop_autotune_counters3[0]};

/* Container */
qtm_freq_hop_autotune_control_t qtm_freq_hop_autotune_control3
= {&qtm_freq_hop_autotune_data3, &qtm_freq_hop_autotune_config3};

/**********************************************************/
/*********************** Keys Module **********************/
/**********************************************************/

/* Keys set 3 - General settings */
qtm_touch_key_group_config_t qtlib_key_grp_config_set3 = {DEF_NUM_SENSORS_3,
	DEF_TOUCH_DET_INT_3,
	DEF_MAX_ON_DURATION_3,
	DEF_ANTI_TCH_DET_INT_3,
	DEF_ANTI_TCH_RECAL_THRSHLD_3,
	DEF_TCH_DRIFT_RATE_3,
	DEF_ANTI_TCH_DRIFT_RATE_3,
	DEF_DRIFT_HOLD_TIME_3,
DEF_REBURST_MODE_3};

qtm_touch_key_group_data_t qtlib_key_grp_data_set3;

/* Key data */
qtm_touch_key_data_t qtlib_key_data_set3[DEF_NUM_SENSORS_3];

/* Key Configurations */
qtm_touch_key_config_t qtlib_key_configs_set3[DEF_NUM_SENSORS_3] = {KEY_0_PARAMS_3, KEY_1_PARAMS_3};

/* Container */
qtm_touch_key_control_t qtlib_key_set3
= {&qtlib_key_grp_data_set3, &qtlib_key_grp_config_set3, &qtlib_key_data_set3[0], &qtlib_key_configs_set3[0]};

/*============================================================================
static touch_ret_t touch_sensors_config(void)
------------------------------------------------------------------------------
Purpose: Initialization of touch key sensors
Input  : none
Output : none
Notes  :
============================================================================*/
/* Touch sensors config - assign nodes to buttons / wheels / sliders / surfaces / water level / etc */
static touch_ret_t touch_sensors_config(void)
{
	uint16_t    sensor_nodes;
	touch_ret_t touch_ret = TOUCH_SUCCESS;

	/* Init acquisition module */
	qtm_ptc_init_acquisition_module(&qtlib_acq_set1);
	qtm_ptc_init_acquisition_module(&qtlib_acq_set2);
	qtm_ptc_init_acquisition_module(&qtlib_acq_set3);

	/* Init pointers to DMA sequence memory */
	//qtm_ptc_qtlib_assign_signal_memory(&touch_acq_signals_raw1[0]);

	/* Initialize sensor nodes */

	for (sensor_nodes = 0u; sensor_nodes < DEF_NUM_CHANNELS_1; sensor_nodes++) {
		/* Enable each node for measurement and mark for calibration */
		qtm_enable_sensor_node(&qtlib_acq_set1, sensor_nodes);
		qtm_calibrate_sensor_node(&qtlib_acq_set1, sensor_nodes);
	}
	
	for (sensor_nodes = 0u; sensor_nodes < DEF_NUM_CHANNELS_2; sensor_nodes++) {
		/* Enable each node for measurement and mark for calibration */
		qtm_enable_sensor_node(&qtlib_acq_set2, sensor_nodes);
		qtm_calibrate_sensor_node(&qtlib_acq_set2, sensor_nodes);
	}

	for (sensor_nodes = 0u; sensor_nodes < DEF_NUM_CHANNELS_3; sensor_nodes++) {
		/* Enable each node for measurement and mark for calibration */
		qtm_enable_sensor_node(&qtlib_acq_set3, sensor_nodes);
		qtm_calibrate_sensor_node(&qtlib_acq_set3, sensor_nodes);
	}

	/* Enable sensor keys and assign nodes */
	for (sensor_nodes = 0u; sensor_nodes < DEF_NUM_CHANNELS_1; sensor_nodes++) {
		qtm_init_sensor_key(&qtlib_key_set1, sensor_nodes, &ptc_qtlib_node_stat1[sensor_nodes]);
	}

	/* Enable sensor keys and assign nodes */
	for (sensor_nodes = 0u; sensor_nodes < DEF_NUM_CHANNELS_2; sensor_nodes++) {
		qtm_init_sensor_key(&qtlib_key_set2, sensor_nodes, &ptc_qtlib_node_stat2[sensor_nodes]);
	}

	/* Enable sensor keys and assign nodes */
	for (sensor_nodes = 0u; sensor_nodes < DEF_NUM_CHANNELS_3; sensor_nodes++) {
		qtm_init_sensor_key(&qtlib_key_set3, sensor_nodes, &ptc_qtlib_node_stat3[sensor_nodes]);
	}

	return (touch_ret);
}

/*============================================================================
static void qtm_measure_complete_callback( void )
------------------------------------------------------------------------------
Purpose: this function is called after the completion of
         measurement cycle. This function sets the post processing request
         flag to trigger the post processing.
Input  : none
Output : none
Notes  :
============================================================================*/
static void qtm_measure_complete_callback(void)
{
	touch_ret_t touch_ret = TOUCH_SUCCESS;
	touch_ret = qtm_acquisition_process();
	if(touch_ret != TOUCH_SUCCESS)
	{
		while(1);
	}
	
	if(acq_set == 1)
	{
		// Configuration 1
		acq_set = 2;					// run Configuration 2 next
		time_to_measure_touch_flag = 1;
	}
	else if (acq_set == 2)
	{
		// Configuration 2
		acq_set = 3;					// run Configuration 3 next
		touch_postprocess_request = 1u;
	}
	else
	{
		// Configuration 3
		acq_set = 1;					// run Configuration 1 next
		touch_postprocess_request = 1u;
	}
}

/*============================================================================
static void qtm_error_callback(uint8_t error)
------------------------------------------------------------------------------
Purpose: this function is used to report error in the modules.
Input  : error code
Output : decoded module error code
Notes  :
Derived Module_error_codes:
    Acquisition module error =1
    post processing module1 error = 2
    post processing module2 error = 3
    ... and so on
============================================================================*/
static void qtm_error_callback(uint8_t error)
{
	module_error_code = error + 1u;

#if DEF_TOUCH_DATA_STREAMER_ENABLE == 1
	datastreamer_output();
#endif
}

/*============================================================================
void touch_init(void)
------------------------------------------------------------------------------
Purpose: Initialization of touch library. PTC, timer, and
         datastreamer modules are initialized in this function.
Input  : none
Output : none
Notes  :
============================================================================*/
void touch_init(void)
{

	touch_timer_config();

	/* Configure touch sensors with Application specific settings */
	touch_sensors_config();

#if DEF_TOUCH_DATA_STREAMER_ENABLE == 1
	datastreamer_init();
#endif
}

/*============================================================================
void touch_process(void)
------------------------------------------------------------------------------
Purpose: Main processing function of touch library. This function initiates the
         acquisition, calls post processing after the acquistion complete and
         sets the flag for next measurement based on the sensor status.
Input  : none
Output : none
Notes  :
============================================================================*/
void touch_process(void)
{
	touch_ret_t touch_ret = TOUCH_SUCCESS;

	/* check the time_to_measure_touch_flag flag for Touch Acquisition */
	if (time_to_measure_touch_flag == 1u) {
		/* Do the acquisition */
		gpio_set_pin_level(TEST, true);	// check timing
		
		if(acq_set == 1)
		{
			qtm_ptc_qtlib_assign_signal_memory(&touch_acq_signals_raw1[0]);
			touch_ret = qtm_ptc_start_measurement_seq(&qtlib_acq_set1, qtm_measure_complete_callback);
		}
		else if(acq_set == 2)
		{
 			qtm_ptc_qtlib_assign_signal_memory(&touch_acq_signals_raw2[0]);
 			touch_ret = qtm_ptc_start_measurement_seq(&qtlib_acq_set2, qtm_measure_complete_callback);
		}
		else if(acq_set == 3)
		{
			qtm_ptc_qtlib_assign_signal_memory(&touch_acq_signals_raw3[0]);
			touch_ret = qtm_ptc_start_measurement_seq(&qtlib_acq_set3, qtm_measure_complete_callback);
		}

		/* if the Acquisition request was successful then clear the request flag */
		if (TOUCH_SUCCESS == touch_ret) {
			/* Clear the Measure request flag */
			time_to_measure_touch_flag = 0u;
		}
	}

	/* check the flag for node level post processing */
	if (touch_postprocess_request == 1u) {
		/* Reset the flags for node_level_post_processing */
		touch_postprocess_request = 0u;

		/* Run Acquisition module level post processing*/
		//touch_ret = qtm_acquisition_process();

		/* Check the return value */
		if (TOUCH_SUCCESS == touch_ret) {
			
			/* Returned with success: Start module level post processing */
			touch_ret = qtm_freq_hop_autotune(&qtm_freq_hop_autotune_control1);
			if (TOUCH_SUCCESS != touch_ret) {
				qtm_error_callback(1);
			}
			touch_ret = qtm_key_sensors_process(&qtlib_key_set1);
			if (TOUCH_SUCCESS != touch_ret) {
				qtm_error_callback(2);
			}

			/* Returned with success: Start module level post processing */
			touch_ret = qtm_freq_hop_autotune(&qtm_freq_hop_autotune_control2);
			if (TOUCH_SUCCESS != touch_ret) {
				qtm_error_callback(1);
			}
			touch_ret = qtm_key_sensors_process(&qtlib_key_set2);
			if (TOUCH_SUCCESS != touch_ret) {
				qtm_error_callback(2);
			}

			/* Returned with success: Start module level post processing */
			touch_ret = qtm_freq_hop_autotune(&qtm_freq_hop_autotune_control3);
			if (TOUCH_SUCCESS != touch_ret) {
				qtm_error_callback(1);
			}
			touch_ret = qtm_key_sensors_process(&qtlib_key_set3);
			if (TOUCH_SUCCESS != touch_ret) {
				qtm_error_callback(2);
			}
		} else {
			/* Acq module Eror Detected: Issue an Acq module common error code 0x80 */
			qtm_error_callback(0);
		}

		if ((0u != (qtlib_key_set1.qtm_touch_key_group_data->qtm_keys_status & 0x80u))) {
			time_to_measure_touch_flag = 1u;
		}
		else if ((0u != (qtlib_key_set2.qtm_touch_key_group_data->qtm_keys_status & 0x80u))) {
			time_to_measure_touch_flag = 1u;
		}
		else if ((0u != (qtlib_key_set3.qtm_touch_key_group_data->qtm_keys_status & 0x80u))) {
			time_to_measure_touch_flag = 1u;
		}
		else {
			measurement_done_touch = 1u;
		}
		
		gpio_set_pin_level(TEST, false);	// check timing

#if DEF_TOUCH_DATA_STREAMER_ENABLE == 1
		datastreamer_output();
#endif
	}
}

/*============================================================================
void touch_timer_handler(void)
------------------------------------------------------------------------------
Purpose: This function updates the time elapsed to the touch key module to
         synchronize the internal time counts used by the module.
Input  : none
Output : none
Notes  :
============================================================================*/
void touch_timer_handler(void)
{
	/* Count complete - Measure touch sensors */
	time_to_measure_touch_flag = 1u;
	qtm_update_qtlib_timer(DEF_TOUCH_MEASUREMENT_PERIOD_MS);
}

void Timer_task_cb(const struct timer_task *const timer_task)
{
	touch_timer_handler();
}

void touch_timer_config(void)
{
	static struct timer_task Timer_task;
#if (KRONO_GESTURE_ENABLE == 1u)
	Timer_task.interval = 1;
#else
	Timer_task.interval = DEF_TOUCH_MEASUREMENT_PERIOD_MS;
#endif
	Timer_task.cb   = Timer_task_cb;
	Timer_task.mode = TIMER_TASK_REPEAT;

	timer_add_task(&Timer, &Timer_task);
	timer_start(&Timer);
}

uint16_t get_sensor_node_signal(uint16_t sensor_node)
{
	return (ptc_qtlib_node_stat1[sensor_node].node_acq_signals);
}

void update_sensor_node_signal(uint16_t sensor_node, uint16_t new_signal)
{
	ptc_qtlib_node_stat1[sensor_node].node_acq_signals = new_signal;
}

uint16_t get_sensor_node_reference(uint16_t sensor_node)
{
	return (qtlib_key_data_set1[sensor_node].channel_reference);
}

void update_sensor_node_reference(uint16_t sensor_node, uint16_t new_reference)
{
	qtlib_key_data_set1[sensor_node].channel_reference = new_reference;
}

uint16_t get_sensor_cc_val(uint16_t sensor_node)
{
	return (ptc_qtlib_node_stat1[sensor_node].node_comp_caps);
}

void update_sensor_cc_val(uint16_t sensor_node, uint16_t new_cc_value)
{
	ptc_qtlib_node_stat1[sensor_node].node_comp_caps = new_cc_value;
}

uint8_t get_sensor_state(uint16_t sensor_node)
{
	return (qtlib_key_set1.qtm_touch_key_data[sensor_node].sensor_state);
}

uint8_t get_sensor_state1(uint16_t sensor_node)
{
	return (qtlib_key_set1.qtm_touch_key_data[sensor_node].sensor_state);
}

uint8_t get_sensor_state2(uint16_t sensor_node)
{
	return (qtlib_key_set2.qtm_touch_key_data[sensor_node].sensor_state);
}

uint8_t get_sensor_state3(uint16_t sensor_node)
{
	return (qtlib_key_set3.qtm_touch_key_data[sensor_node].sensor_state);
}

void update_sensor_state(uint16_t sensor_node, uint8_t new_state)
{
	qtlib_key_set1.qtm_touch_key_data[sensor_node].sensor_state = new_state;
}

void calibrate_node(uint16_t sensor_node)
{
	/* Calibrate Node */
	qtm_calibrate_sensor_node(&qtlib_acq_set1, sensor_node);
	/* Initialize key */
	qtm_init_sensor_key(&qtlib_key_set1, sensor_node, &ptc_qtlib_node_stat1[sensor_node]);
}

/*============================================================================
void PTC_Handler_EOC(void)
------------------------------------------------------------------------------
Purpose: Interrupt service handler for PTC EOC interrupt
Input  : none
Output : none
Notes  : none
============================================================================*/
void PTC_Handler(void)
{
	qtm_ptc_clear_interrupt();
	qtm_samd20_ptc_handler_eoc();
}

#endif /* TOUCH_C */
